import processing.core.*; 
import processing.xml.*; 

import com.jsyn.engine.SynthesisEngine; 
import com.jsyn.unitgen.LineIn; 
import com.jsyn.unitgen.PeakFollower; 
import com.jsyn.devices.AudioDeviceManager; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class VoicePain extends PApplet {


/*

 VoicePain
 
 02 march 2012
 
 FOR NATE!!!
 
 */






SynthesisEngine synth;
LineIn lineIn;
PeakFollower follower;

boolean jsynRunning = false;

int x = 0;
int y = 0;

int xStep = 20;
int yStep = 20;

int rb = 127;
int bb = 127;
int gb = 127;
int ab = 200;

int rf = 127;
int bf = 127;
int gf = 127;
int af = 127;


static int LRTB = 0;
static int HLINE = 1;
static int RECTS = 2;

int MODE = HLINE;

long millisStart = 0;


public void setup()
{
  size(screen.width, screen.height);
  smooth();

  //Create a context for the synthesizer.
  synth = new SynthesisEngine();

  //Add an output mixer.
  //synth.add(lineOut = new LineOut());
  synth.add( lineIn = new LineIn() );

  synth.add(follower = new PeakFollower());
  follower.halfLife.set(0.001f);
  lineIn.output.connect(follower.input);

  synth.start( 44100, AudioDeviceManager.USE_DEFAULT_DEVICE, 2, 0, 0);

  follower.start();

  jsynRunning = true;

  background(0);
  stroke(255);

  millisStart = millis();
}

public void draw()
{

  if (millis() - millisStart > 30000)
  {

    if (random(0, 1) < 0.5f)
    {
      int newMode = (int)(random(0, 3));

      MODE = newMode;
    }
    millisStart = millis();
  }


  double peakAmplitude = follower.current.getValue();

  if (MODE == LRTB)
  {
    fill(rb, gb, bb, ab);
    stroke(rf, gf, bf, af);
    
    ellipse(x, y, (int)(100 * peakAmplitude), (int)(100 * peakAmplitude));

    x += xStep;

    if (x >= width)
    {
      y += yStep;
      x = width - 1;
      xStep *= -1;
    }
    else if (x <= 0)
    {
      y += yStep;
      x = 0;
      xStep *= -1;
    }

    if (y >= height)
    {
      y = height - 1;
      yStep *= -1;
    }
    else if (y < 0)
    {
      y = 0;
      yStep *= -1;
    }
  }


  else if (MODE == HLINE)
  {
    fill(rb, gb, bb, ab);
    stroke(rf, gf, bf, af);
    
    int jump = (int)(75 * peakAmplitude);
    rect(0, y, width, 5 + jump);

    y += jump;

    if (y >= height)
      y = 0;

  }

  else if (MODE == RECTS)
  {
    peakAmplitude += 0.2f;
    fill(rb, gb, bb, ab);
    stroke(rf, gf, bf, af);

    int w = (int)(random(0, 1) * peakAmplitude * 100);
    int h = (int)(random(0, 1) * peakAmplitude * 100);

    rect(x, y, w, h);

    int xJump = (int)(random(-1, 1) * peakAmplitude * 100);
    int yJump = (int)(random(-1, 1) * peakAmplitude * 100);

    x += xJump;
    y += yJump;

    if (x < 0 || x >= width)
      x -= xJump;

    if (y < 0 || y >= height)
      y -= yJump;
  }


    int rOffset = (int)(random(-16, 16) * peakAmplitude);
    if (rb + rOffset <= 255 && rb + rOffset >= 0)
      rb += rOffset;

    int gOffset = (int)(random(-16, 16) * peakAmplitude);
    if (gb + gOffset <= 255 && gb + gOffset >= 0)
      gb += gOffset;

    int bOffset = (int)(random(-16, 16) * peakAmplitude);
    if (bb + bOffset <= 255 && bb + bOffset >= 0)
      bb += bOffset;

    int aOffset = (int)(random(-16, 16) * peakAmplitude);
    if (ab + aOffset <= 255 && ab + aOffset >= 127)
      ab += aOffset;


    rOffset = (int)(random(-16, 16) * peakAmplitude);
    if (rf + rOffset <= 255 && rf + rOffset >= 0)
      rf += rOffset;

    gOffset = (int)(random(-16, 16) * peakAmplitude);
    if (gf + gOffset <= 255 && gf + gOffset >= 0)
      gf += gOffset;

    bOffset = (int)(random(-16, 16) * peakAmplitude);
    if (bf + bOffset <= 255 && bf + bOffset >= 0)
      bf += bOffset;

    aOffset = (int)(random(-16, 16) * peakAmplitude);
    if (af + aOffset <= 255 && af + aOffset >= 127)
      af += aOffset;

}


public void stop() {
  print("killing jsyn...");
  synth.stop();
  print("bye.");
}

public void keyPressed() 
{

  background(0);
  int newMode = (MODE + 1) % 3;

  MODE = newMode;
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#FFFFFF", "VoicePain" });
  }
}
